#include "MorphoAibo.h"
#include <queue>

using namespace std;

const unsigned char MorphoAibo::DEFAULT_KERNEL[3][3]={
  {1,1,1},
  {1,1,1},
  {1,1,1},
};

MorphoAibo::MorphoAibo(const unsigned char* image, int width, int height)
  : isValid(true),
    mImageSize(width*height),
    mImageWidth(width),
    mImageHeight(height)
{
  mImage=new unsigned char[mImageWidth*mImageHeight];
  memcpy(mImage, image, mImageSize);
  memcpy(mKernel, DEFAULT_KERNEL, KERNEL_SIZE);
}

MorphoAibo::~MorphoAibo(){
  if(isValid)
    {
      if(mImage != 0)
	delete mImage;
    }
}

void MorphoAibo::loadImage(const unsigned char* image, int width, int height){
  isValid=true;
  mImageSize=width*height;
  mImageWidth=width;
  mImageHeight=height;
  mImage=new unsigned char[mImageWidth*mImageHeight];
  memcpy(mImage, image, mImageSize);
  memcpy(mKernel, DEFAULT_KERNEL, KERNEL_SIZE);
}

size_t MorphoAibo::getImage(unsigned char* retImage){
  if(!isValid)
    return 0;
  
  memcpy(retImage, mImage, mImageSize);
  return mImageSize;
}

void MorphoAibo::erode(unsigned char mask){
  // Se non valida esco.
  if(!isValid)
    return;

  queue<unsigned char*> setToZero;
  
  size_t imageSize=mImageWidth * mImageHeight;

  unsigned char* minPtr=mImage;
  unsigned char* maxPtr=mImage+imageSize - 1;
  unsigned char* imagePtr=mImage;
  
  unsigned char* tempPtr;
  //
  // Scorro l'immagine
  //
  for(size_t i=0; i < imageSize; i++)
    {
      // Se il pixel e' di foreground
      if(*imagePtr & mask)
	{
	  bool skip=false;
	  
	  //
	  // Sovrappogno il Kernel ad ogni pixel.
	  //
	  for(int ii=0; !skip && ii < 3 ; ii++)
	    for(int jj=0; jj < 3; jj++)
	      {
		if(mKernel[ii][jj] == 1)
		  {
		    tempPtr=imagePtr - mImageWidth + ( mImageWidth * ii ) - 1 + jj;
		    
		    // Se il pixel esiste allora controllalo
		    if(tempPtr >= minPtr && tempPtr <= maxPtr)
		      if( !( *tempPtr & mask) )
			{
			  setToZero.push(imagePtr);
			  skip=true;
			  break;
			}
		  }
	      }
	} // Fine scansione pixel
      
      imagePtr++;
    } // Fine scansione immagine

  //
  // Modifico l'immagine.
  // Setto a zero i pixel individuati.
  while(!setToZero.empty())
    {
      tempPtr=setToZero.front();
      setToZero.pop();
      // uso Xor che effettua il toggle del pixel
      *tempPtr= (*tempPtr) ^ mask;
    }
}
  
void MorphoAibo::dilate(unsigned char mask){
  // Se non valida esco.
  if(!isValid)
    return;

  queue<unsigned char*> setToOne;
  
  size_t imageSize=mImageWidth * mImageHeight;

  unsigned char* minPtr=mImage;
  unsigned char* maxPtr=mImage+imageSize - 1;
  unsigned char* imagePtr=mImage;
  
  unsigned char* tempPtr;
  //
  // Scorro l'immagine
  //
  for(size_t i=0; i < imageSize; i++)
    {
      // Se il pixel e' di background
      if(! (*imagePtr & mask) )
	{
	  bool skip=false;
	  
	  //
	  // Sovrappogno il Kernel ad ogni pixel.
	  //
	  for(int ii=0; !skip && ii < 3 ; ii++)
	    for(int jj=0; jj < 3; jj++)
	      {
		if(mKernel[ii][jj] == 1)
		  {
		    tempPtr=imagePtr - mImageWidth + ( mImageWidth * ii ) - 1 + jj;
		    
		    // Se il pixel esiste allora controllalo
		    if(tempPtr >= minPtr && tempPtr <= maxPtr)
		      if( *tempPtr & mask )
			{
			  setToOne.push(imagePtr);
			  skip=true;
			  break;
			}
		  }
	      }

	} // Fine scansione pixel
      
      imagePtr++;
    } // Fine scansione immagine

  //
  // Modifico l'immagine.
  // Setto a zero i pixel individuati.
  while(!setToOne.empty())
    {
      tempPtr=setToOne.front();
      setToOne.pop();
      // uso Xor che effettua il toggle del pixel
      *tempPtr= (*tempPtr) | mask;
    }
}

