#include <iostream>
#include "MorphoAibo.h"
#include <string>
#include "write_jpeg.h"

using namespace std;

static const int gDefaultWidth=104;
static const int gDefaultHeight=80;

static const string FILE_NAME="image";

const unsigned char CDTMixedColor[]={ 128,128,128};
/** Colori utilizzati per colorare i CDT channel*/
const unsigned char CDTChannelColor[][3]={
  {0,128,128},     // no color detected = black
  {149,22,48},     // channel 0 = light green
  {29,108,255},    // channel 1 = blue
  {105, 234,218},  // channel 2 = pink
  {76,255,86},     // channel 3 = red
  {255, 128, 128}, // channel 4 = white
  {255,149,1},     // channel 5 = yellow
  {75,75,86},      // channel 6 = dark green 
  {140,185,49},    // channel 7 = orange
};

unsigned char CDTColorMatrix[256][3];
void PutYCbCrPixel(unsigned char* img, int w,
		   int x, int y, unsigned char ypix, unsigned char cb, unsigned char cr) {
  unsigned char* ptr = img + 3 * (w * y + x);
  ptr[0] = ypix;
  ptr[1] = cb;
  ptr[2] = cr;
}

void initCDTMatrix(){
  CDTColorMatrix[0][0]=CDTChannelColor[0][0];
  CDTColorMatrix[0][1]=CDTChannelColor[0][1];
  CDTColorMatrix[0][2]=CDTChannelColor[0][2];

  int actualColor=1;

  for(int i=1; i< 256; i++)
    {
    if(i & actualColor)
      {
	CDTColorMatrix[i][0]=CDTChannelColor[i][0];
	CDTColorMatrix[i][1]=CDTChannelColor[i][1];
	CDTColorMatrix[i][2]=CDTChannelColor[i][2];
	actualColor*=2;
      }
    else
      {
	CDTColorMatrix[i][0]=CDTMixedColor[0];
	CDTColorMatrix[i][1]=CDTMixedColor[1];
	CDTColorMatrix[i][2]=CDTMixedColor[2];
      }     
    }
  
}

void ReconstructAndConvertCtd(unsigned char* imageVec,unsigned char* image,
			      int width, int height)
{
//   OFbkImageInfo* info = imageVec->GetInfo(layer);
//   unsigned char*          data = imageVec->GetData(layer);

//   OFbkImage cdtImage(info, data, ofbkimageBAND_CDT);

//   int w     = cdtImage.Width();
//   int h     = cdtImage.Height();
//   unsigned char* ptr = cdtImage.Pointer();

//   *width  = 2 * w;
//   *height = 2 * h;
  
  unsigned char* ptr=imageVec;

  char yValue,crValue,cbValue;
  
  for (int y = 0; y < height; y++) 
    {
      for (int x = 0; x < width; x++) 
	{
	  yValue=CDTColorMatrix[*ptr][0];
	  crValue=CDTColorMatrix[*ptr][1];
	  cbValue=CDTColorMatrix[*ptr][2];
	  
	  PutYCbCrPixel(image, width, x,   y,   yValue, cbValue, crValue);
// 	  PutYCbCrPixel(image, width, 2*x+1, 2*y,   yValue, cbValue, crValue);
// 	  PutYCbCrPixel(image, width, 2*x,   2*y+1, yValue, cbValue, crValue);
// 	  PutYCbCrPixel(image, width, 2*x+1, 2*y+1, yValue, cbValue, crValue);
	  ptr++;
	}
    }
  return;
}

int main(){
  //
  // Data un immagine in formato raw cdt presa dal cane il programma
  // dovrebbe: salvare una jpeg contenente l'immagine originale con i
  // colori riconosciuti, una jpeg contenente l'immagine sottoposta a
  // open e l'immagine sottoposta a close.
  //
  
  string rawImage="./" + FILE_NAME + ".raw";
  string originalImage="./" + FILE_NAME + "original.jpg";
  string openImage="./" + FILE_NAME + "open.jpg";
  string closeImage="./" + FILE_NAME + "close.jpg";

  initCDTMatrix();

  MorphoAibo mfi;
  mfi.open(0x01);
  
  unsigned char originalBuffer[gDefaultWidth * gDefaultHeight];
  unsigned char recostructedBuffer[ (gDefaultWidth * gDefaultHeight) * 3 ];
  FILE* tempFile;
  tempFile=fopen(rawImage.c_str(), "r");
  if(tempFile == NULL)
    {
      cerr << "Errore di apertuara file 1" << endl;
      return -1;
    }
  fread(originalBuffer, 1, gDefaultWidth * gDefaultHeight, tempFile);
  fclose(tempFile);

  //
  // Salvo l'immagine originale
  //
  ReconstructAndConvertCtd(originalBuffer, recostructedBuffer, gDefaultWidth, gDefaultHeight);
  tempFile=fopen(originalImage.c_str(), "w");
  if(tempFile == NULL)
    {
      cerr << "Errore di apertuara file 2" << endl;
      return -1;
    }
  write_jpeg_file(recostructedBuffer, gDefaultWidth, gDefaultHeight, 95, tempFile);
  fclose(tempFile);

  MorphoAibo mfopen(originalBuffer, gDefaultWidth, gDefaultHeight);
  MorphoAibo mfclose(originalBuffer, gDefaultWidth, gDefaultHeight);
  //
  // Salvo l'immagine aperta
  // 
  mfopen.open(0x01);
  mfopen.getImage(originalBuffer);
  ReconstructAndConvertCtd(originalBuffer, recostructedBuffer, gDefaultWidth, gDefaultHeight);
  tempFile=fopen(openImage.c_str(), "w");
  if(tempFile == NULL)
    {
      cerr << "Errore di apertuara file 3" << endl;
      return -1;
    }
  write_jpeg_file(recostructedBuffer, gDefaultWidth, gDefaultHeight, 95, tempFile);
  fclose(tempFile);

  //
  // Salvo l'immagine chiusa
  //
  cout << "chiudo" << endl;
  mfclose.close(0x01);
  mfclose.close(0x02);
  mfclose.close(0x04);
  mfclose.close(0x08);
//   cout << "apro" << endl;
//   mfclose.open(0x01);
  mfclose.getImage(originalBuffer);
  ReconstructAndConvertCtd(originalBuffer, recostructedBuffer, gDefaultWidth, gDefaultHeight);
  tempFile=fopen(closeImage.c_str(), "w");
  if(tempFile == NULL)
    {
      cerr << "Errore di apertuara file 4" << endl;
      return -1;
    }
  write_jpeg_file(recostructedBuffer, gDefaultWidth, gDefaultHeight, 95, tempFile);
  fclose(tempFile);

  //aprire rawImage
  // convertirla
  // salvarla come originalImage
  // creare MorphoAibo
  // applicare la apertura
  // salvarla come openImage
  
  // creare un'altro MorphoAibo
  // applicare la chiusura
  // salvarla come closeImage.

  //write_jpeg_file(immagine, width, height, 10, FILE*);

  
  return 0;
}
